open Transitions
open Term
open Definitions


(* vieux trucs 
type automaton_irr = {a: Automaton.automaton; joker: state; absorbant: state}

*)




type airr_prod = {
	prod: Automaton.Produit.produit;
	airr :Automaton.automaton;
	correspondants_q : (state, Stateset.t) Hashtbl.t;
}


let correspondants (self: airr_prod) (q: state) : Stateset.t =
	try
		Hashtbl.find self.correspondants_q q
	with
		| Not_found -> Stateset.empty

let embed_hook_airr (self: airr_prod) (q: state) (p: state) (qp: state) (ajout: bool) : unit =
 if ajout then (
			let ls = correspondants self q
			in
				Hashtbl.replace self.correspondants_q q (Stateset.add qp ls)
		);;



let access_hook_airr (self:airr_prod) (q: state) (p: state) (qp: state) : unit =
	let corres = correspondants self q in
		Hashtbl.replace self.correspondants_q q (Stateset.remove qp corres);;



let final_crit_airr (self: airr_prod) (q: state) (p: state) : bool =
	Stateset.mem p self.airr.Automaton.fstates;;


let init_airr_prod (a0: Automaton.automaton) (airr: Automaton.automaton) =
	let resultat = {prod={Automaton.Produit.a=Automaton.make (); Automaton.Produit.memo_embed = Hashtbl.create 0; Automaton.Produit.memo_embedrev =  Hashtbl.create 0}; airr = airr; correspondants_q =  Hashtbl.create 0;}
	in let aprod = Automaton.Produit.init_produit ~accessibility_hook:(access_hook_airr resultat) ~embed_hook:(embed_hook_airr resultat) ~final_crit:(final_crit_airr resultat) a0 airr
	in {resultat with prod=aprod};;
	
	
(** [add_symtrans self (f, sl, dest)] ajoute dans le produit self la transition f(sl)→dest de A *)
let add_symtrans (self: airr_prod) ((f, sl, dest) : external_symtrans) : unit =
		let ltrans_airr = simple_get_trans_f f self.airr.Automaton.trans
		and prod_t_t (f1, sl0, dest0) (f2, slirr, destirr) = (* on aura f1=f2*)
				Transitions.add self.prod.Automaton.Produit.a.Automaton.trans (f1, Automaton.Produit.embed_list ~hook:(embed_hook_airr self) self.prod sl0 slirr, Automaton.Produit.embed ~hook:(embed_hook_airr self) self.prod dest0 destirr);
				if Stateset.mem destirr self.airr.Automaton.fstates then self.prod.Automaton.Produit.a.Automaton.fstates <- Stateset.add (Automaton.Produit.embed self.prod dest0 destirr) self.prod.Automaton.Produit.a.Automaton.fstates
		in List.iter (prod_t_t (f, sl, dest)) ltrans_airr;;
		 

let add_Rtrans (self: airr_prod) ((q, q', phi): external_epsilon) : unit =	
	let corres = correspondants self q
	and prod_un p =
		Transitions.addR self.prod.Automaton.Produit.a.Automaton.trans (Automaton.Produit.embed self.prod q p, Automaton.Produit.embed self.prod q' p, phi)
	in
		Stateset.iter prod_un corres

let add_Etrans (self: airr_prod) ((q, q'): state * state) : unit =	
		let corres = Stateset.elements (correspondants self q)
		(*and corres' = Stateset.elements (correspondants self q')*)
		in let lp = List.map (Automaton.Produit.proj_p self.prod) corres
		(*and lp' = List.map (Automaton.Produit.proj_p self.prod) corres'*)
		and prod_un p =
			Transitions.addE self.prod.Automaton.Produit.a.Automaton.trans (Automaton.Produit.embed self.prod q p) (Automaton.Produit.embed self.prod q' p)
		in
			List.iter prod_un lp(*;
			List.iter prod_un lp'*)
			

type epsi_is_e_or_r = E | R;;

let fstsnd (a,b,c) = (a,b);;

let add_translist (self: airr_prod) (transl : external_trans list) (epsi_t : epsi_is_e_or_r) : unit =
	let add_one trans =
		match trans with
		 | Trans(tt) -> add_symtrans self tt
			| 	Epsilon(tt) -> (match epsi_t with
								| R -> () (* nous sommes convenus de ne pas ajouter les εR au produit *)
								| E -> add_Etrans self (fstsnd tt))
	in
		List.iter add_one transl;;
		 
(**
@param t est un terme clos (pas une configuration) *)
let is_a_normal_form (self: airr_prod) (t: term) : bool =
	try
		let reconnait_t = Automaton.all_run self.airr.Automaton.trans t in
			List.exists (fun p -> Stateset.mem p self.airr.Automaton.fstates) reconnait_t
	with
		_ -> Verbose.println 0 (fun _ -> "Warning: Airr.is_a_normal_form("^(Term.to_string t)^"): true by exception; computation may be imprecise"); true

(**
@param cqp est une configuration de l'automate produit *)
let recognises_normal_forms (self: airr_prod) (cqp: term) : bool =
	Verbose.println 6 (fun _ -> "recognises_normal_forms: cqp = "^ (Automaton.Produit.cqp_to_string self.prod cqp));
	try
		let reconnait_t = Automaton.all_run self.prod.Automaton.Produit.a.Automaton.trans cqp in
			List.exists (Automaton.Produit.is_final self.prod) reconnait_t
	with
		_ -> Verbose.println 0 (fun _ -> "Warning: Airr.recognises_normal_forms("^(Automaton.Produit.cqp_to_string self.prod cqp)^"): true by exception; computation may be imprecise"); true

(** [expand_config self cq] renvoie la liste des configurations cqp possibles en partant de cq *)
let rec expand_config (self: airr_prod) (cq : term) : term list = match cq with
	| State q -> List.map (fun x -> State x) (Stateset.elements (correspondants self q))
	| Fsym (f, cl) -> let cel = expand_config_list self cl and
		ajouter_un (rem: term list list) (couple: term) : term list list =
			let res = List.map (fun l -> couple::l) rem in
				Verbose.println 6 (fun _ -> "exp_cfg: ajouter_un: rem = " ^ (List2.to_string (List2.to_string (Automaton.Produit.cqp_to_string self.prod)) rem) ^"\n      couple = "^(Automaton.Produit.cqp_to_string self.prod couple)^"\n     res = "^(List2.to_string (List2.to_string (Automaton.Produit.cqp_to_string self.prod)) res));
				res
		in let ajouter_etat_etendu (l_extensions: term list) (rem: term list list) : term list list =
			List.fold_left (fun acc couple -> (ajouter_un rem couple)@acc) [] l_extensions
		and construire (ll : term list list) : term list =
			List.map (fun sl -> Fsym(f, sl)) ll
		in let le_produit = List.fold_right ajouter_etat_etendu cel [[]]
		in construire le_produit
	| Var _ -> failwith "Airr.expand_config: configuration non close"
		
and expand_config_list (self: airr_prod) (cql : term list) : term list list =
	List.map (expand_config self) cql;;
		
let recognises_normal_forms_q (self: airr_prod) (cq : term) : bool =
	Verbose.println 5 (fun _ -> "recognises_normal_forms_q: cq = "^ (Term.to_string cq));
	let lcqp = expand_config self cq in
		Verbose.println 5 (fun _ -> "recognises_normal_forms_q: lcqp = " ^ (List2.to_string (Automaton.Produit.cqp_to_string self.prod) lcqp));
		List.exists (recognises_normal_forms self) lcqp;;



let int_nf_crit a airr x xirr =
		let fa = (Stateset.mem x a.Automaton.fstates)
		and fairr = (Stateset.mem xirr airr.Automaton.fstates)
		in fa && fairr

let inter_nf (a: Automaton.automaton) (airr: Automaton.automaton) : Automaton.automaton =
	
	let adet = Automaton.export (Automaton.export_epsi a) in (*Automaton.Determinisation.determinise a in*)
	let prod = Automaton.Produit.init_produit ~final_crit:(int_nf_crit adet airr) adet airr
	in  let res =( (prod.Automaton.Produit.a))
	in
		Verbose.println 7 (fun _ -> Automaton.Produit.prodtrans_to_string prod);
		snd (Automaton.renumerotation (Automaton.utility_cleaning res))			
		
(* vieux trucs			
(** teste l'égalité membre à membre de deux listes, avec une tolérance : à chaque position, l'élément de la deuxième liste peut être un joker
		utilisé pour vérifier qu'une transition f(q_1;…;q_n) comportant des jokers est applicable à to_what
*)
let list_equal_or_joker (autom: automaton_irr) (to_what: state list) (trans: state list) : bool =
	try
		List.for_all2 (fun qref qtrans -> qref = qtrans || qtrans = autom.joker) to_what trans
	with
		Invalid_argument _ -> false (* ne devrait pas se produire car on est censé appeler cette fonction seulement sur des listes d'états associés à un symbole donné, donc de mêmes longeurs *)
		
let is_absorbant (autom: automaton_irr) (t: term) : bool = match t with
	| State q -> q = autom.absorbant
	| _ -> false;;

exception Configuration_not_recognised of term
exception Automaton_is_not_deterministic of term

(* Deterministic run of t, with no epsilon resolving *)
let rec det_run_ja (autom: automaton_irr) (t: term) : term =
	let d = autom.a.Automaton.trans in
		match t with
  | Fsym (f, []) ->
      (match Transitions.get_trans_sym_constant f d with
 	 | [] -> raise (Configuration_not_recognised (Fsym (f, []))) (* B: le cas ou il n'y a aucune transition *)
	 | [(_, _, q)] -> State q 
   | _ -> raise (Automaton_is_not_deterministic  (Fsym (f, [])))                              
      )
  | Fsym (f, tl) ->
		  let res_tl = List.map (det_run_ja autom) tl in
			if List.exists (is_absorbant autom) res_tl then  (* s'il y a un état absorbant, on le renvoie *)
					State autom.absorbant
			else (
    	   let res_states = List.map Term.get_state res_tl in
    	   let relevant_trans =
    	     List.flatten
                   (List.map
                      (fun q -> Transitions.get_trans_sym_parameter f q d)
    		  res_states
                   )
               in (* on commence par chercher une transition comme dans le cas général *)
    	       	(match List.find_all (fun  (_, sl ,_) -> sl = res_states) relevant_trans with
            		| [] -> (* s'il n'y en a pas, on cherche avec les jokers *)
											(match List.find_all (fun  (_, sl ,_) -> list_equal_or_joker autom res_states sl) relevant_trans with
												| [] -> raise (Configuration_not_recognised (Fsym (f, res_tl)))
												| [_, _, q] -> State q
												| _ -> raise (Automaton_is_not_deterministic (Fsym(f, res_tl)))
											)
    				 		| [_, _, q] ->   State q
    						| _ -> raise (Automaton_is_not_deterministic (Fsym(f, res_tl)))
    						)
				)

  | State q -> State q
  | Var _ -> failwith "Automaton:oriented_run: non ground term!";;
		

(*(* Non deterministic run: use any transition to normalize a term (with epsilonE resolving only).
   Used for application of equations: given a \sigma and an equation l=r produces, for l\sigma all
   possible states reachable, modulo epsilonE. *)
(* gives every possible state in which a term can be rewritten or the empty list *)
(* Principle is to find all transitions rooted by the same top symbol (all
   f(q1...qn)->q for f(t1...tn) and then removing parameter states q...qn of transitions
   one after one, and keep only transitions such that they were matching t1...tn successively *)

(* if l\sigma -> q and q=q' then, by default, we don't give q' as a "reachable" state, since it will be added
   as a successor by E of all new predecessors of q by transitivity closure of epsilon E. More precisely,
   if r\sigma q'', then we only give the result q for l\sigma. Since all_run is only used for equation application
   we have l\sigma -> q and r\sigma -> q'' and q <->q'. By adding q <->q'' we add (by transitivity) q'' <-> q'.
   Hence, no need to give q' as a result.
 *)
let rec all_run (a: Automaton.automaton) (airr: Automaton.automaton) (t: term) (t_airr: term) : (state * state) list =
	let d = a.Automaton.trans and dirr = airr.Automaton.trans in
  (*  let _= print_string ("Term= "^(Term.to_string t)^"\n") in *)
  match t, t_airr with
    | Fsym (f, []), Fsym(g, []) when f = g -> (* TODO*)
        (match Transitions.get_trans_sym_constant f d with
	   | [] -> []
	   | l -> [](*List.map (fun  (_, _, q) -> q) l*) 
        )
    | Fsym (f, tl), Fsym(g, tl_airr) when f = g ->
        all_run_list a airr tl tl_airr (Transitions.simple_get_trans_f f d) (Transitions.simple_get_trans_f g dirr)
    | State q, State p -> [(q,p)]
    | Var "_", _ ->
        failwith "Automaton:all_run: anonymous variable cannot be used on top!"
    | Var _, _ ->
        failwith "Automaton:all_run: free variable uninstantiated!"
          (* all_run_list recursively run through all transitions of l
             and either remove the 1st parameter of each transition if
             it is accepted, or reject the complete transition *)
          (* B: Reprendre ici !!! *)
          (* La fonction all_run_list execute tous les run possible
             sur les sous-termes en parallele *)

(* print_string ("Term list= "^(String.concat ", " (List.map Term.to_string tl))^
   "\non l=" ^ (Transitions.symextlist_to_string l)^"\n"); *)
(** @param l est typiquement la liste de toutes les transitions de d ayant un certain symbole comme tête, par exemple f
@param tl est typiquement la liste des sous-termes sous le symbole f dans le terme dont all_run calcule les descendants
@return une liste d'états qui sera renvoyée par all_run, donc ce doit être la liste des états en lesquels f(tl) est reconnu *)
and all_run_list (a: Automaton.automaton) (airr:Automaton.automaton) (tl: term list) (tl_airr: term list) (l: external_symtrans list) (l_airr: external_symtrans list) : (state * state) list =
	let d = a.Automaton.trans and dirr = airr.Automaton.trans in
  let remain_trans =  (* on va éliminer les transitions qui ne vérifient pas certains criètres… *)
    List.fold_left
      (fun ltrans subterm -> (* external_symtrans list -> term -> external_symtrans list *)
          let states = all_run a airr subterm in (* liste des états en lesquels subterm est reconnu *)
                 List2.optmap  (* effet global du bazar sur la liste ltrans :
								pour chaque « transition » f(q1,…,qn) → dest, on regarde si q1 [mod E] reconnait subterm 
								si tel n'est pas le cas, la transition est éliminée
								si tel est bien le cas, on ampute la transition de q1 pour en faire la pseudo-transition f(q2,…,qn) → dest
								Cette amputation du premier état permet de conserver à chaque étape du fold_left la propriété que
								subterm = t_i, notant tl=[t_1,…,t_k], et le premier état de chaque pseudo-transition de ltrans est le q_i de la vraie transition dont elle est issue*)
	           (fun ((f, sl, dest), (g, sl_airr, dest_airr)) -> (* trans -> state option *) (* en fait, ce n'est pas forcément une vraie transition*) 
                      let q' = List.hd sl and p'= List.hd sl_airr in (* sl ne peut pas être vide : au départ, les listes d'états dans les transitions de l ont toutes la même longueur que tl ; à chaque étape du fold_left, on avance dans tl et on décapite les listes d'états des transitions ; à la fin on vide complètement ces listes, mais alors le fold_left est fini *)
                        (* if q' is an Esucc of one state q of sl then it
                           is ok, i.e.  t -> Delta+E q ->E q' *)
		        if
                          List.exists
                            (fun q -> q = q' || (Transitions.is_succE d q' q)) (* égalité mod E *)
                            states
                        then (* dans states, il y a un état égal [mod E] à l'état en tête de sl
													ie. l'état en tête de sl est parmi [mod E] ceux en lesquels subterm est reconnu *)
                          Some (f, List.tl sl, dest) (* Achtung ! ceci ressemble à une transition, mais n'en n'est plus vraiment une car le nombre de sous termes sous t est incorrect*)
		        else
                          None
                   ) ltrans
      ) l tl
  in
    List.map (fun (_, _, dest) -> dest) remain_trans*)
*)

let rec variable_abstraction (t: term) : term = match t with
	| Var _ -> Var "_"
	| State(q) -> State(q)
	| Fsym(f, tl) -> let tla = List.map variable_abstraction tl in Fsym(f, tla)

let make_does_not_contain_normal_forms (prod : airr_prod) :
 (term -> bool) = fun t -> not (recognises_normal_forms_q prod t);;