open Definitions

type automaton = {
  trans : Transitions.t;
  mutable fstates : Stateset.t;
}

val make : unit -> automaton


val state_is_final : automaton -> Term.term -> bool
val det_run : Transitions.t -> Term.term -> Term.term

val transtable : automaton -> Transitions.transtable
val copy : automaton -> automaton
val external_to_TA : Stateset.t -> Transitions.external_trans list -> automaton
val to_string : automaton -> string
val to_timbuk : automaton -> string



type s2t = (state, Term.term) Hashtbl.t
val new_s2t : unit -> ('a, 'b) Hashtbl.t
val s2t_to_string : (int, Term.term) Hashtbl.t -> string


type statevar = (state, Definitions.variable) Hashtbl.t

val new_statevar : unit -> ('a, 'b) Hashtbl.t
val statevar_add : statevar -> state -> Definitions.variable -> unit
val statevar_find : ('a, 'b) Hashtbl.t -> 'a -> 'b
(** [statevar_find vl q] renvoie ce à quoi [q] est associé dans [vl], ou lève l'exception [Not_found]
 
    Plus précisément, cet appel permet de disciminer les états qui représentent des variables, et le cas échéant de connaitre cette variable *)

val statevar_to_string : (int, string) Hashtbl.t -> string
val statevar_to_list : (state, string) Hashtbl.t -> int list
val statevar_mem : (state, string) Hashtbl.t -> state -> bool


val oriented_run :
  Transitions.t -> Term.term -> state -> bool

val all_run :
  Transitions.t -> Term.term -> state list

val normalize_term :
  Transitions.t ->
  Term.term -> bool -> state * Transitions.external_trans list

val normalize_termE :
  Term.term ->
  Transitions.t ->
  state * Formula.t * Transitions.external_trans list

val normalize_new_states :
  Term.term ->
  Transitions.t -> state * Transitions.external_trans list

val oldnormalize_term :
  bool -> Term.term ->
  state -> Transitions.t -> Transitions.external_trans list

val term_to_automaton :
  Term.term list ->
  symbol list ->
  s2t -> automaton * statevar * state list


val not_empty_intersection : (state * state) list -> Transitions.t -> Formula.t

val inter_of' : automaton -> automaton -> Formula.t
val refine : automaton -> automaton -> int

val init_renum_tabl : automaton -> (state, state) Hashtbl.t
val renumerotation : ?table:((state, state) Hashtbl.t) -> automaton -> ((state, state) Hashtbl.t * automaton)


val accessibility_cleaning : ?hook:(Stateset.t -> unit) -> automaton -> automaton
val utility_cleaning : ?hook:(Stateset.t -> unit) -> automaton -> automaton


val export : automaton -> automaton 
val export_epsi : automaton -> automaton 



module Produit :
sig
	type produit = {
	a: automaton;
	memo_embed : (Definitions.state * Definitions.state, Definitions.state) Hashtbl.t;
	memo_embedrev : (Definitions.state, Definitions.state * Definitions.state) Hashtbl.t;	
	}
	
	val embed: ?hook:(Definitions.state -> Definitions.state -> Definitions.state -> bool -> unit) -> produit -> Definitions.state -> Definitions.state -> Definitions.state
	val embed_list: ?hook:(Definitions.state -> Definitions.state -> Definitions.state -> bool -> unit) -> produit -> Definitions.state list -> Definitions.state list -> Definitions.state list
	val proj_p : produit -> Definitions.state -> Definitions.state
	
	val is_final : produit -> Definitions.state -> bool
	
	val qp_to_string : produit -> Definitions.state -> string
	val cqp_to_string : produit -> Term.term -> string
	val prodtrans_to_string : produit -> string
	val init_produit : ?accessibility_hook:(Definitions.state -> Definitions.state -> Definitions.state -> unit) -> ?embed_hook:(Definitions.state -> Definitions.state -> Definitions.state -> bool -> unit) -> final_crit:(Definitions.state -> Definitions.state -> bool) -> automaton -> automaton -> produit
	
end

module Determinisation :
sig
	val determinise : automaton -> automaton
end







