open Definitions
open Term
open Transitions
open Automaton

(* Simple dot output function for completion *)
(* producing dot edges *)

let string_of_rel (q, es) =
  Stateset.fold 
    (fun q2 s2 -> (State.to_string q) ^ " -> " ^ 
       (State.to_string q2) (*^ ";\n"*) ^ s2) es ""
  

(* A child of q2 is q if there exists a transition f(..., q, ...) -> q2 *)
let children_of (q: state) (delta: t) : Stateset.t =
  let rec children_of (acc: Stateset.t) = function
    | [] -> acc
    | (_, sl ,_)::tl -> children_of (Stateset.add_list sl acc) tl
  in
    children_of Stateset.empty (Transitions.simple_get_all_trans [q] delta)

let sonrel (sts: Stateset.t) (d: Transitions.t): (state * Stateset.t) list =
  Stateset.fold
    (fun s l ->
       let sons = children_of s d in	 
	 (s, sons)::l
    )
    sts []

let tempEpsi_to_string =
  List2.to_string ~sep:"\n" ~left:"" ~right:"" 
    (fun (q1, q2) -> (State.to_string q1) ^ "->" ^ (State.to_string q2))
    

let write_dot (filename: string) (acomp: Automaton.automaton) (select: term list) : unit =
  let fullepsilist = (* Give epsilon transitions but without closure *)
    Hashtbl2.fold
      (fun s1 es acc ->
         Stateset.fold (fun s2 acc -> (s1, s2)::acc) es acc
      ) acomp.trans.epsilonR#succs []
  in
    (* States corresponding to states matched by select pattern *)
  let statelist = 
    List.flatten (List.map (Automaton.all_run acomp.trans) select) 
  in
    Verbose.println 3
      (fun _ -> "List of initial states to write to dot file: " ^ 
         (List2.to_string ~sep:"," ~left:"" ~right:"" 
            State.to_string statelist
         ) ^ "\n"
      );
    (* We get all the predecessors of states w.r.t. epsilon transitions,
       i.e. descendants w.r.t. rewriting *)
    let stateset = 
      List.fold_right 
        (fun s set ->
	   Stateset.add_list (s::(fst(List.split (Transitions.get_predRE acomp.trans s)))) set)
        statelist Stateset.empty 
    in
      Verbose.println 3
        (fun _ -> "List of all states to write to dot file: " ^ (Stateset.to_string stateset));
      Verbose.println 3
        (fun _ -> "Epsilist before filtering: " ^ (tempEpsi_to_string fullepsilist));
      let epsilist =
        List.filter 
          (fun (q1, q2) -> 
             Stateset.mem q1 stateset || Stateset.mem q2 stateset
          ) fullepsilist 
      in

        Verbose.println 3
          (fun _ -> "Epsilist after filtering: " ^ (tempEpsi_to_string epsilist));
        let sonrel = 
          if !Common.dotson then sonrel stateset acomp.trans else []
        in
          Verbose.println 3
            (fun _ -> List2.to_string ~sep:"\n" ~left:"" ~right:""
               (fun (s, es) -> "sonrel=" ^ 
                  (State.to_string s) ^ "," ^ (Stateset.to_string es)
               ) sonrel
            );
          let resstring = 
            "digraph compdot {\n" ^
              (List2.to_string ~sep:";\n" ~left:"" ~right:"\n"
	         (fun (q1, q2) -> (State.to_string q2)^" -> "^(State.to_string q1))
	         epsilist
              ) ^ 
              "edge [color=brown3];\n" ^
              (List2.to_string ~sep:";\n" ~left:"" ~right:"\n"
	         string_of_rel sonrel
              ) ^ "}" 
          in
          let outchan = open_out filename in
            try
              Printf.fprintf outchan "%s" resstring;
              close_out outchan
            with exc -> close_out outchan; raise exc
