val maxVar : int ref
module Buddy :
  sig
    type error =
        BDD_MEMORY
      | BDD_VAR
      | BDD_RANGE
      | BDD_DEREF
      | BDD_RUNNING
      | BDD_FILE
      | BDD_FORMAT
      | BDD_ORDER
      | BDD_BREAK
      | BDD_VARNUM
      | BDD_NODES
      | BDD_OP
      | BDD_VARSET
      | BDD_VARBLK
      | BDD_DECVNUM
      | BDD_REPLACE
      | BDD_NODENUM
      | BDD_ILLBDD
      | BDD_SIZE
      | BVEC_SIZE
      | BVEC_SHIFT
      | BVEC_DIVZERO
    exception BddError of error
    external init : int -> int -> int = "ocaml_bdd_init"
    type t
    type gbc_stat = {
      nodes : int;
      freenodes : int;
      time : float;
      sumtime : float;
      num : int;
    }
    external gbc_hook : (bool -> gbc_stat -> unit) -> unit
      = "ocaml_bdd_gbc_hook"
    external var : t -> int = "ocaml_bdd_var"
    external low : t -> t = "ocaml_bdd_low"
    external high : t -> t = "ocaml_bdd_high"
    external support : t -> t = "ocaml_bdd_support"
    type value = X | F | T
    external allsat : ('acc -> value array -> 'acc) -> 'acc -> t -> 'acc
      = "ocaml_bdd_allsat"
    external satcount : t -> float = "ocaml_bdd_satcount"
    external satcountset : t -> t -> float = "ocaml_bdd_satcountset"
    external satcountsetln : t -> float = "ocaml_bdd_satcountln"
    external satcountlnset : t -> t -> float = "ocaml_bdd_satcountlnset"
    external nodecount : t -> int = "ocaml_bdd_nodecount"
    external anodecount : t -> int = "ocaml_bdd_anodecount"
    external pathcount : t -> float = "ocaml_bdd_pathcount"
    external reset : unit -> unit = "ocaml_bdd_done"
    external setvarnum : int -> int = "ocaml_bdd_setvarnum"
    external extvarnum : int -> int = "ocaml_bdd_extvarnum"
    external isrunning : unit -> bool = "ocaml_bdd_isrunning"
    external setcacheratio : int -> int = "ocaml_bdd_setcacheratio"
    external setmaxnodenum : int -> int = "ocaml_bdd_setmaxnodenum"
    external setmaxincrease : int -> int = "ocaml_bdd_setmaxincrease"
    external setminfreenodes : int -> int = "ocaml_bdd_setminfreenodes"
    external getnodenum : unit -> int = "ocaml_bdd_getnodenum"
    external getallocnum : unit -> int = "ocaml_bdd_getallocnum"
    external versionstr : unit -> string = "ocaml_bdd_versionstr"
    external versionnum : unit -> int = "ocaml_bdd_versionnum"
    external tt : unit -> t = "ocaml_bdd_true"
    external ff : unit -> t = "ocaml_bdd_false"
    external varnum : unit -> int = "ocaml_bdd_varnum"
    external ithvar : int -> t = "ocaml_bdd_ithvar"
    external nithvar : int -> t = "ocaml_bdd_nithvar"
    external gbc : unit -> unit = "ocaml_bdd_gbc"
    external not : t -> t = "ocaml_bdd_not"
    type bddop =
        AND
      | XOR
      | OR
      | NAND
      | NOR
      | IMP
      | BIIMP
      | DIFF
      | LESS
      | INVIMP
    external apply : t -> t -> bddop -> t = "ocaml_bdd_apply"
    external ite : t -> t -> t -> t = "ocaml_bdd_ite"
    external restrict : t -> t -> t = "ocaml_bdd_restrict"
    external constrain : t -> t -> t = "ocaml_bdd_constrain"
    type vars =
        (int32, Bigarray.int32_elt, Bigarray.c_layout) Bigarray.Array1.t
    external replace_all : t -> vars -> vars -> t = "ocaml_bdd_replace_all"
    external replace : t -> int -> int -> t = "ocaml_bdd_replace"
    external compose : t -> t -> t -> t = "ocaml_bdd_compose"
    external exist : t -> t -> t = "ocaml_bdd_exist"
    external forall : t -> t -> t = "ocaml_bdd_forall"
    external unique : t -> t -> t = "ocaml_bdd_unique"
    external appex : t -> t -> bddop -> t -> t = "ocaml_bdd_appex"
    external appall : t -> t -> bddop -> t -> t = "ocaml_bdd_appall"
    external swapvar : int -> int -> int = "ocaml_bdd_swapvar"
    type reorder =
        WIN2
      | WIN2ITE
      | WIN3
      | WIN3ITE
      | SIFT
      | SIFTITE
      | RANDOM
      | NONE
    external reorder : reorder -> unit = "ocaml_bdd_reorder"
    external reorder_gain : unit -> int = "ocaml_bdd_reorder_gain"
    external clrvarblocks : unit -> unit = "ocaml_bdd_clrvarblocks"
    external intaddvarblock : t -> bool -> int = "ocaml_bdd_intaddvarblock"
    external varblockall : unit -> unit = "ocaml_bdd_varblockall"
    external autoreorder : reorder -> reorder = "ocaml_bdd_autoreorder"
    external autoreorder_times : reorder -> int -> reorder
      = "ocaml_bdd_autoreorder_times"
    external var2level : int -> int = "ocaml_bdd_var2level"
    external enable_reorder : bool -> unit
      = "ocaml_bdd_enable_disable_reorder"
    external fdd_overlapdomain : int -> int -> int
      = "ocaml_fdd_overlapdomain"
    external fdd_clearall : unit -> unit = "ocaml_fdd_clearall"
    external fdd_domainnum : unit -> int = "ocaml_fdd_domainnum"
    external fdd_equals : int -> int -> int = "ocaml_fdd_equals"
    external fdd_extdomain : vars -> int = "ocaml_fdd_extdomain"
    external fdd_intaddvarblock : int -> int -> int
      = "ocaml_fdd_intaddvarblock"
    external fdd_ithset : int -> t = "ocaml_fdd_ithset"
    external fdd_ithvar : int -> int -> t = "ocaml_fdd_ithvar"
    external fdd_vars : int -> int array = "ocaml_fdd_vars"
  end
val gc_handler : bool -> Buddy.gbc_stat -> unit
val atom_to_string : int * int * Definitions.kind -> string
val vars2eq : (int, Definitions.atom) Hashtbl.t
val eq2vars : (Definitions.atom, int) Hashtbl.t
type formula = Buddy.t
type t = formula
val currentVar : int ref
val free_vars : int list ref
val mk_true : Buddy.t
val mk_false : Buddy.t
val atom_of_var : int -> Definitions.atom
val var_of_atom : Definitions.atom -> int
val mk_atom :
  Definitions.state -> Definitions.state -> Definitions.kind -> formula
val kill_atom :
  Definitions.state -> Definitions.state -> Definitions.kind -> unit
val mk_and : formula -> formula -> formula
val mk_or : formula -> formula -> formula
val mk_not : formula -> formula
val rm_atom : formula -> Definitions.atom -> formula
val unsat : formula -> bool
val tautology : formula -> bool
val subsume : formula -> formula -> bool
val equival : Buddy.t -> Buddy.t -> bool
val is_sat : formula -> bool
val to_list : Buddy.value array -> (Definitions.atom * bool) list
val allsat : formula -> (Definitions.atom * bool) list list
val negatives : formula -> Definitions.atom list list
val valuation_to_string : (Definitions.atom * bool) list list -> string
val to_string : formula -> string
