type pTerm =
    (* | Joker (* former PVar "_" *) *)
  | PVar of string
  | PFsym of string * pTerm list
  | PState of string


let completeTF = ref false
let specialState = "qTF"

exception SpecialStateDeclaration
exception SpecialStateInEpsilon
exception SpecialStateAsRHS

(* Pour definir plus finement les types de termes et donc les types qui suivent je propose

   (* Ceux-ci n'est pas forcement necessaire... *)
   type pJoker = [ `PJoker ]

   type pVar = [ `PVar of string ]

   type pState = [ `PState of string ]

   type 'a pFsym = [ `PFsym of string * 'a list ]

   type pTerm = [ pTerm pFsym | pVar | pJoker ]

        (* ou explicitement les constructeurs de pVar et pJoker *)

   type pConf = [ pConf pFsym | pState ]

   type pClosed = pClosed pFsym

   type pAll = [ pAll pFsym | pState | pVar | pJoker ]

   let rec to_string : pAll -> string = function
   | `PVar x -> x
   | `PState q -> q
   | `PJoker -> "_"
   | `PFsym (f, []) -> f
   | `PFsym (f, terms) ->
   f ^ (List2.to_string ~sep:", " ~left:"(" ~right:")" terms)

   Example of usage

   (* [tm] is defined and explicitly typed with the smallest type: *)
   let tm : pClosed = `PFsym ("f", [ `PFsym ("g", [ `PFsym ("a", []); `PFsym ("a", []) ]);
                      `PFsym ("g", [ `PFsym ("a", []); `PFsym ("a", []) ]) ]);;

   The expression [to_string tm] is not well-typed, except if you remove all the type
   anotations of the definitions of [to_string] and [tm].

   So you need explicit coercion to recover well-typing of [tm] applied to [to_string],
   i.e. you have to coerce type [pClosed] into [pAll]. This coercion is written
   [(tm : pClosed :> pAll)] or [(tm :> pAll) ] more simply. Finally, the expression
   [to_string (tm :> pAll)] is now well-typed.

   It is also possible to define coercion function:

   [let pAll_of_pClosed (term: pClosed) = (term :> pAll)]
   or
   [let pAll_of_pClosed term = (term : pClosed :> pAll)]

*)


type pCondition =
  | PEqual of (pTerm * pTerm)
  | PJoinable of (pTerm * pTerm)
  | PNotEqual of (pTerm * pTerm)


type pEquation =
  | PEquation of pTerm * pTerm * pCondition list
  | PExteq of pTerm * pTerm * (pTerm * pTerm) list
  | PUnique of pTerm * (pTerm * pTerm) list
  | PAny of pTerm * pTerm * (pTerm * pTerm) list


type pRule =
  | Usual of pTerm * pTerm
  | Conditionnal of pTerm * pTerm * pCondition list

(*
  type pAutomaton =
  | PAut of
      string * pState list * pState list * (pConf * pState) list * (pConf * pState) list
  | PSet of string * pConst list

*)


type pAutomaton =
  | PAut of
      string * pTerm list * pTerm list * (pTerm * pTerm) list * (pTerm * pTerm) list
  | PSet of string * pTerm list

type pApprox =
  | PApp of
      string * pTerm list * ((pTerm * pTerm) * (pTerm * pTerm) list) list
  | PSEquation of string * pEquation list

(* Nouvelle spec
   type timbukspec =
     (string * pRule list) list *
       pAutomaton list *
       pTerm list * pTerm list *
       pApprox list
*)


type timbukspec =
    (string * pRule list) list *
      pAutomaton list *
      pTerm list * pTerm list *
      pApprox list

module PTerm = struct
  type t = pTerm

  let compare = compare

  let equal = (=)


  let rec to_string = function
    | PVar x -> x
    | PState x -> x
    | PFsym (f, []) -> f
    | PFsym (f, lt) ->
        f ^ "(" ^ (List2.to_string ~sep:"," to_string lt) ^ ")"

(*
  let rec to_string = function
    | `PVar x -> x
    | `PState q -> q
    | `PFsym (f, []) -> f
    | `PFsym (f, lt) ->
        f ^ "(" ^ (List2.to_string "," to_string lt)^")"
*)
end


let alpha: (string * int) list ref = ref []
let varset: string list ref = ref []


