include Array

(* Returns a truncated copy of [t = [| x0; ...; xn |]], if [dim < length t].
   Otherwise, it returns a new array, where each appended cell
   are filled by using [f] parametrized by index of the cell:
   [| x0; ...; xn; f (n + 1); ...; f (dim - 1) |]
*)

(* optional_argument = 
   `Val of int 
   `Fun of int -> int
*)

let resize ?default t dim  = 
  if dim < 0 || dim > Sys.max_array_length then
    raise (Invalid_argument "Array2.resize");
  let v0 = match default with
    | None -> if length t <> 0 then t.(0) else
        raise (Failure "Array2.resize")
          (* Il faut absolument une valeur pour redimensionner [||] *)
    | Some (`Val v) -> v
    | Some (`Fun f) -> f 0
  in
  let t' = Array.make dim v0
  and last = min (length t) dim - 1 in
    for i = 0 to last do
      t'.(i) <- t.(i)
    done;
    match default with 
      | Some (`Fun f) -> 
          for i = last + 1 to dim - 1 do
            t'.(i) <- f i
          done;
          t'
      | _ -> t'

let resize_matrix ?default m d1 d2 = 
  if d1 < 0 || d1 > Sys.max_array_length ||
    d2 < 0 || d2 > Sys.max_array_length
  then
    raise (Invalid_argument "Array2.resize_matrix");
  let v0 = match default with
    | None ->
        if length m > 0 && (length m.(0) > 0) then 
          m.(0).(0)
        else
          raise (Failure "Array2.resize")
            (* Il faut absolument une valeur pour redimensionner [||] *)
    | Some (`Val v) -> v
    | Some (`Fun f) -> f 0 0
  in
  let m' = Array.make_matrix d1 d2 v0
  and l1 = min (length m) d1 - 1 
  in 
    for i = 0 to l1 do
      for j = 0 to min (length m.(i)) d2 - 1 do
        m'.(i).(j) <- m.(i).(j)
      done
    done;
    match default with
      | Some (`Fun f) ->
          for i = l1 + 1 to d1 - 1 do
            for j = min (length m.(i)) d2 to d2 - 1 do
              m'.(i).(j) <- f i j
            done
          done;
          m'
      | _ -> m'

           
