include Dumper

let asserteq s1 s2 s =
  if s1 <> s2 then
    failwith ("Equality assertion failure on: " ^ s ^ ": " ^ 
                (dump s1) ^ "  and  " ^ (dump s2) )

let assertneq s1 s2 s =
  if s1 = s2 then
    failwith ("Disequality assertion failure on: " ^ s ^ ": " ^ 
                (dump s1) ^ "  and  " ^ (dump s2) )


(** Removes the unnecessary white spaces that occurs at the beginning
    or the end of the string *)
let rec remove_spaces (str: string) : string =
  let len = String.length str in
    if len = 0 then
      str
    else if str.[0] = ' ' then
      remove_spaces (String.sub str 1 (len - 1))
    else if String.get str (len - 1) = ' ' then
      remove_spaces (String.sub str 0 (len - 1))
    else
      str

(** Splits a string with '\n' (by default) into a list of substrings.
    Removes unecessary white spaces at the beginning or end for each
    substring.  You can use any different separator by invoking
    [strcut ~sep:'$' str], whereas invoking [strcut str] uses the default
    separator ('\n')
*)

let rec strcut ?(sep: char = '\n') (str: string) : string list =
  let strcut = strcut ~sep:sep in
  let len = String.length str in
    try
      let i = String.index str sep in
        if i = 0 then
          if len = 1 then
            []
          else 
            strcut (String.sub str 1 (len - 1))
        else
          let first = remove_spaces (String.sub str 0 i) in
            if first = "" then
              strcut (String.sub str i (len - i))
            else 
              first::(strcut (String.sub str i (len - i)))
    with Not_found ->
      let first = remove_spaces str in
        if first = "" then [] else [first]
          

(** Basically checks that a string is equal (modulo order) to an other string.
    For pretty printing comparison (in unit testing) without taking order of
    LINES into account. *)
let string_equal (s1: string) (s2: string) =
  let rec aux_equal l1 l2 : bool =
    match l1, l2 with
      | [], [] -> true
      | e1::rem1, l2 -> 
          if List.mem e1 l2 then
            aux_equal rem1 (List2.remove e1 l2)
          else (
            print_string ("lhs " ^ e1 ^ " is not in the rhs\n");
            false 
          )
      | [], e2::_ ->
          print_string ("rhs " ^ e2 ^ " is not in the lhs\n");
          false
  in
    aux_equal (strcut s1) (strcut s2)

(************************************ TESTS ***********************************)
IFDEF TEST THEN

let s2=
"States q4 q3 q2 q1 q0  
Final States q4  
Transitions  
y -> q2
x -> q1
f(q0,q3) -> q4
a -> q0
g(q1,q2) -> q3
"
let s2b =
"States q4 q3 q2 q1 q0  
Final States q4
Transitions
f(q0,q3) -> q4
g(q1,q2) -> q3
y -> q2
x -> q1
a -> q0"

let _TEST_ = 
  asserteq (remove_spaces "  toto    ") "toto" "remove_spaces";           
  asserteq (strcut "toto    \n   \ntiti\n\n    tata\n\n") 
    ["toto";"titi";"tata"] "string2stringlist";
  asserteq (string_equal "\n\ntoto\n\ntata\ntiti\n" 
              "toto    \n   \ntiti\n\n    tata\n\n") true "string_equal";
  asserteq (string_equal "\n\na b  \n" "b a") false "string_equal2";
  asserteq (string_equal s2 s2b) true "string_equal3"

END
