(** This module extends module Set of the standard library by adding
    a function "to_string" to dump the structure. Each element of the dumped
    set is convert to a string using a function [to_string] provided by
    a module of type [OrderedType], which is required by the functor [Make].
*)

module type OrderedType =
sig
  type t

  val compare : t -> t -> int
  val to_string : t -> string
end


module type S =
  sig
    include Set.S

    val add_list : elt list -> t -> t
    val to_string :
      ?sep:string -> ?left:string -> ?right:string ->
      ?str_elt:(elt -> string) -> t -> string

    val replace : elt -> elt -> t -> t
  end


module Make (Ord : OrderedType) : S with type elt = Ord.t
