{
(* ..................................................................
    Module.......: Lexer
    Initial Date : 21 may 2007
    Last Update..: 21 may 2007
    Developer....: Yohan Boichut
    Maintenance..: Yohan Boichut and Thomas Genet
    Compiler.....: OCAML 3.09.3
    Comments.....: Lexer for Timbuk
 .................................................................... *)

  open Parser

  let kwdTbl = Hashtbl.create 20
  let _ =
    Hashtbl.add kwdTbl "Ops"	OPS;
    Hashtbl.add kwdTbl "Vars"	VARS;
    Hashtbl.add kwdTbl "TRS"	TRS;
    Hashtbl.add kwdTbl "Set"	SET;
    Hashtbl.add kwdTbl "Prior"	PRIOR;
    Hashtbl.add kwdTbl "States"	STATES;
    Hashtbl.add kwdTbl "Final"  FINAL;
    Hashtbl.add kwdTbl "Rules"	RULES;
    Hashtbl.add kwdTbl "if"	IF;
    Hashtbl.add kwdTbl "equal"	EQUAL;
    Hashtbl.add kwdTbl "Patterns"	PATTERN;
    Hashtbl.add kwdTbl "SubPatterns"    SUBPATTERN;
    Hashtbl.add kwdTbl "Automaton"	AUTOMATON;
    Hashtbl.add kwdTbl "Transitions"	TRANSITIONS;
    Hashtbl.add kwdTbl "Equations"	EQUATIONS;
    Hashtbl.add kwdTbl "Approximation"  APPROX;
    Hashtbl.add kwdTbl "DotPatterns"    DOTPAT

  let keyWord str =
    try
      Hashtbl.find kwdTbl str
    with Not_found -> IDENT str

  (** [depth] stands for the comment depth;
      usefull to control the well balancing of the comments.
  *)
  let depth = ref 0


  (** raised when a bad character is encountered *)
  exception LexError of char

}



let space	= [' ' '\t']
let letter	= ['a'-'z' 'A'-'Z']
let digit	= ['0'-'9']

let ident	= letter (letter | digit | '_' | '-')*
let state       = 'q'(letter | digit | '_' | '-')*
let integer	= digit+

let comline	= '%'([^ '\n'])*

rule token = parse
  | space+		{ token lexbuf }  (* skip blanks *)
  | '\n'        	{ Lexing.new_line lexbuf; token lexbuf }
  | comline     	{ token lexbuf }  (* skip comments *)
  | "/*"                { incr depth; comment lexbuf }
  | ':'                 { COLON }
  | '['                 { LB }
  | ']'                 { RB }
  | '('                 { LP }
  | ')'                 { RP }
  | "->"                { GIVES }
  | "!"                 { NOT }
  | "<->"               { JOIN }
  | " /\\"              { AND }
  | "=>"                { IMP }
  | '='                 { EQU }
  | ','                 { COMMA }
  | state as s          { STATE s }
  | ident as id         { keyWord id }
  | integer as j        { INT (int_of_string j) }
  | '_'                 { SPEVAR }
  | eof            	{ EOF }
  | _ as c  		{ raise (LexError c) }


and comment = parse
  | "/*"		{ incr depth; comment lexbuf}
  | "*/"		{ decr depth;
			  (if !depth > 0 then comment else token)
			    lexbuf
                        }
  | '\n'                { Lexing.new_line lexbuf; comment lexbuf }
  | eof            	{ EOF } (* Un warning serait peut-etre le bienvenu pour indiquer
                                   que les commentaires parses ne sont pas termines... *)
  | _                   { comment lexbuf }
