module Substitution :
  sig
    module Subtemp :
      sig
        type key = Definitions.Variable.t
        type 'a t = 'a Map.Make(Definitions.Variable).t
        val empty : 'a t
        val is_empty : 'a t -> bool
        val mem : key -> 'a t -> bool
        val add : key -> 'a -> 'a t -> 'a t
        val singleton : key -> 'a -> 'a t
        val remove : key -> 'a t -> 'a t
        val merge :
          (key -> 'a option -> 'b option -> 'c option) ->
          'a t -> 'b t -> 'c t
        val compare : ('a -> 'a -> int) -> 'a t -> 'a t -> int
        val equal : ('a -> 'a -> bool) -> 'a t -> 'a t -> bool
        val iter : (key -> 'a -> unit) -> 'a t -> unit
        val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
        val for_all : (key -> 'a -> bool) -> 'a t -> bool
        val exists : (key -> 'a -> bool) -> 'a t -> bool
        val filter : (key -> 'a -> bool) -> 'a t -> 'a t
        val partition : (key -> 'a -> bool) -> 'a t -> 'a t * 'a t
        val cardinal : 'a t -> int
        val bindings : 'a t -> (key * 'a) list
        val min_binding : 'a t -> key * 'a
        val max_binding : 'a t -> key * 'a
        val choose : 'a t -> key * 'a
        val split : key -> 'a t -> 'a t * 'a option * 'a t
        val find : key -> 'a t -> 'a
        val map : ('a -> 'b) -> 'a t -> 'b t
        val mapi : (key -> 'a -> 'b) -> 'a t -> 'b t
      end
    type substitution = Term.term Subtemp.t
    type t = substitution
    val empty : 'a Subtemp.t
    val add : Subtemp.key -> 'a -> 'a Subtemp.t -> 'a Subtemp.t
    val equal : ('a -> 'a -> bool) -> 'a Subtemp.t -> 'a Subtemp.t -> bool
    val find : Subtemp.key -> 'a Subtemp.t -> 'a
    val fold : (Subtemp.key -> 'a -> 'b -> 'b) -> 'a Subtemp.t -> 'b -> 'b
    val map : ('a -> 'b) -> 'a Subtemp.t -> 'b Subtemp.t
    val compare : substitution -> substitution -> int
    val cardinal : 'a Subtemp.t -> int
    val union : 'a Subtemp.t -> 'a Subtemp.t -> 'a Subtemp.t
    val to_string : substitution -> string
    exception Inclusion
    val covers : substitution -> substitution -> bool
    val is_included : substitution -> substitution -> bool
    val extended_equivalence : substitution -> substitution -> bool
  end
type substitution = Substitution.t
module SubstitutionPhi :
  sig
    type t = { value : substitution; phi : Formula.t; }
    exception Inclusion
    val empty : t
    val add : Substitution.Subtemp.key -> Term.term -> t -> t
    val define : substitution -> Formula.t -> t
    val addPhi : Formula.formula -> t -> t
    val addBoth :
      Substitution.Subtemp.key -> Term.term -> Formula.formula -> t -> t
			(** [addBoth x t φ σ] renvoie la substitution-avec-formule issue de [σ] avec [.value] augmentée par x ↦ t et [.phi = σ.phi ∨ φ] *)
			
    val compare : t -> t -> int
    val cardinal : t -> int
    val union : t -> t -> t
    val to_string : t -> string
    val coversValue : t -> t -> bool
    val covers : t -> t -> bool
    val equal : t -> t -> bool
    val is_included : t -> t -> bool
    val extended_equivalence : t -> t -> bool
  end
type substitutionPhi = SubstitutionPhi.t
module SubstitutionSet :
  sig
    module Substsettemp :
      sig
        type key = Substitution.t
        type 'a t = 'a Map.Make(Substitution).t
        val empty : 'a t
        val is_empty : 'a t -> bool
        val mem : key -> 'a t -> bool
        val add : key -> 'a -> 'a t -> 'a t
        val singleton : key -> 'a -> 'a t
        val remove : key -> 'a t -> 'a t
        val merge :
          (key -> 'a option -> 'b option -> 'c option) ->
          'a t -> 'b t -> 'c t
        val compare : ('a -> 'a -> int) -> 'a t -> 'a t -> int
        val equal : ('a -> 'a -> bool) -> 'a t -> 'a t -> bool
        val iter : (key -> 'a -> unit) -> 'a t -> unit
        val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
        val for_all : (key -> 'a -> bool) -> 'a t -> bool
        val exists : (key -> 'a -> bool) -> 'a t -> bool
        val filter : (key -> 'a -> bool) -> 'a t -> 'a t
        val partition : (key -> 'a -> bool) -> 'a t -> 'a t * 'a t
        val cardinal : 'a t -> int
        val bindings : 'a t -> (key * 'a) list
        val min_binding : 'a t -> key * 'a
        val max_binding : 'a t -> key * 'a
        val choose : 'a t -> key * 'a
        val split : key -> 'a t -> 'a t * 'a option * 'a t
        val find : key -> 'a t -> 'a
        val map : ('a -> 'b) -> 'a t -> 'b t
        val mapi : (key -> 'a -> 'b) -> 'a t -> 'b t
      end
    type substitutionset = Formula.t Substsettemp.t
    type t = substitutionset
    val empty : 'a Substsettemp.t
    val to_list : Formula.t Substsettemp.t -> SubstitutionPhi.t list
    val to_string : Formula.t Substsettemp.t -> string
    val list_to_string : Formula.t Substsettemp.t list -> string
    val mem : SubstitutionPhi.t -> Formula.formula Substsettemp.t -> bool
    val replace :
      Substsettemp.key -> 'a -> 'a Substsettemp.t -> 'a Substsettemp.t
    val add :
      substitutionPhi ->
      Formula.formula Substsettemp.t -> Formula.formula Substsettemp.t
    val fold :
      (Substsettemp.key -> 'a -> 'b -> 'b) -> 'a Substsettemp.t -> 'b -> 'b
    val addPhi_to_all : Formula.t -> substitutionset -> substitutionset
    val from_list : substitutionPhi list -> Formula.formula Substsettemp.t
    val diff :
      substitutionset -> substitutionset -> Formula.formula Substsettemp.t
    val is_empty : 'a Substsettemp.t -> bool
    val filter :
      (substitutionPhi -> bool) ->
      Formula.t Substsettemp.t -> Formula.formula Substsettemp.t
    val included :
      Formula.Buddy.t Substsettemp.t ->
      Formula.Buddy.t Substsettemp.t -> bool
    val equal :
      Formula.Buddy.t Substsettemp.t ->
      Formula.Buddy.t Substsettemp.t -> bool
    val singleton : substitutionPhi -> Formula.formula Substsettemp.t
		(** [singleton s] renvoie le singleton [{s}] *)
		
    val elements : Formula.t Substsettemp.t -> substitutionPhi list
    val union :
      Formula.t Substsettemp.t ->
      Formula.formula Substsettemp.t -> Formula.formula Substsettemp.t
    val combine : substitutionset list -> substitutionset
    val matching_substitutionset : substitutionset -> substitutionset -> bool
    val extended_intersection :
      substitutionset -> substitutionset -> substitutionset
  end
type substitutionset = SubstitutionSet.t
module SubstitutionTable :
  sig
    type substable =
        (Definitions.state, (Definitions.state, substitutionset) Hashtbl.t)
        Hashtbl.t
    type t = substable
    val new_substable : unit -> substable
    exception Couple_not_found
    val find :
      Definitions.state * Definitions.state -> substable -> substitutionset
    val mem :
      Definitions.state * Definitions.state ->
      substitutionPhi -> substable -> bool
					(** [mem (q1, q2) s t] renvoie [s ⋹ t(q1, q2)] *)

    val add :
      substitutionPhi ->
      Definitions.state * Definitions.state -> substable -> bool
    val mark :
      Definitions.state * Definitions.state -> substable -> unit
    val add_set :
      substitutionset ->
      Definitions.state * Definitions.state -> substable -> substitutionset
    val to_string : substable -> string
    val list_to_string :
      ((Definitions.state * Definitions.state) * substitutionset) list ->
      string
    val copy : substable -> substable
  end
type substable = SubstitutionTable.t










